@@ -61,7 +61,7 @@ int main(int argc, char **argv)
 
 	struct starpu_conf conf;
 	starpu_conf_init(&conf);
-	conf.nmic = 0;
+	/* starpu_data_get_user_data cannot work in master-slave */
 	conf.nmpi_ms = 0;
 
 	// initialize StarPU with default configuration
