@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2015                                Université de Bordeaux
- * Copyright (C) 2010-2016                                CNRS
- * Copyright (C) 2013                                     Inria
+ * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -23,8 +21,9 @@
 
 #ifdef STARPU_USE_OPENCL
 struct starpu_opencl_program opencl_code;
-void opencl_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void opencl_codelet(void *descr[], void *_args)
 {
+	(void)_args;
 	cl_kernel kernel;
 	cl_command_queue queue;
 	int id, devid, err, n;
@@ -33,6 +32,7 @@ void opencl_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 	cl_mem mult = (cl_mem)STARPU_VECTOR_GET_DEV_HANDLE(descr[2]);
 	int nx = STARPU_MATRIX_GET_NX(descr[0]);
 	int ny = STARPU_MATRIX_GET_NY(descr[0]);
+	int ld = STARPU_MATRIX_GET_LD(descr[0]);
 
         id = starpu_worker_get_id_check();
         devid = starpu_worker_get_devid(id);
@@ -46,6 +46,7 @@ void opencl_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
         err |= clSetKernelArg(kernel, n++, sizeof(nx), (void*)&nx);
         err |= clSetKernelArg(kernel, n++, sizeof(ny), (void*)&ny);
 	err |= clSetKernelArg(kernel, n++, sizeof(mult), &mult);
+	err |= clSetKernelArg(kernel, n++, sizeof(ld), (void*)&ld);
         if (err) STARPU_OPENCL_REPORT_ERROR(err);
 
 	{
@@ -136,16 +137,14 @@ static struct starpu_codelet cl =
 	.model = &starpu_matvecmult_model
 };
 
-int main(int argc, char **argv)
+int main(void)
 {
-
 	struct starpu_conf conf;
 
 	starpu_conf_init(&conf);
 	conf.ncpus = 0;
 	conf.ncuda = 0;
 	conf.nmic = 0;
-	conf.nscc = 0;
 	conf.nopencl = 1;
 
         /* int width=1100; */
