@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2011       Télécom-SudParis
  * Copyright (C) 2016       Uppsala University
  *
@@ -520,6 +520,20 @@ struct starpu_codelet
 	*/
 	unsigned color;
 
+	/**
+	   Optional field, the default value is <c>NULL</c>. This is a
+	   function pointer of prototype <c>void (*f)(void *)</c>
+	   which specifies a possible callback. If this pointer is
+	   non-<c>NULL</c>, the callback function is executed on the
+	   host after the execution of the task. If the task defines a
+	   callback, the codelet callback is not called, unless called
+	   within the task callback function.
+	   The callback is passed the value contained in the
+	   starpu_task::callback_arg field. No callback is executed if
+	   the field is set to <c>NULL</c>.
+	*/
+	void (*callback_func)(void *);
+
 	/**
 	   Various flags for the codelet.
 	 */
@@ -735,7 +749,7 @@ struct starpu_task
 	   <c>NULL</c>.
 
 	   With starpu_task_insert() and alike this can be specified thanks to
-	   ::STARPU_CALLBACK_ARG followed by the function pointer, or thanks to
+	   ::STARPU_CALLBACK_ARG followed by the argument pointer, or thanks to
 	   ::STARPU_CALLBACK_WITH_ARG or
 	   ::STARPU_CALLBACK_WITH_ARG_NFREE followed by the function
 	   pointer and the argument.
@@ -768,7 +782,28 @@ struct starpu_task
 	*/
 	void *prologue_callback_arg;
 
+	/** Optional field, the default value is <c>NULL</c>. This is a
+	   function pointer of prototype <c>void (*f)(void*)</c>
+	   which specifies a possible callback. If this pointer is
+	   non-<c>NULL</c>, the callback function is executed on the host
+	   when the task is pop-ed from the scheduler, just before getting
+	   executed. The callback is passed the value contained in the
+	   starpu_task::prologue_callback_pop_arg field.
+	   No callback is executed if the field is set to <c>NULL</c>.
+
+	   With starpu_task_insert() and alike this can be specified thanks to
+	   ::STARPU_PROLOGUE_CALLBACK_POP followed by the function pointer.
+	*/
 	void (*prologue_callback_pop_func)(void *);
+	/**
+	   Optional field, the default value is <c>NULL</c>. This is
+	   the pointer passed to the prologue_callback_pop function. This
+	   field is ignored if the field
+	   starpu_task::prologue_callback_pop_func is set to <c>NULL</c>.
+
+	   With starpu_task_insert() and alike this can be specified thanks to
+	   ::STARPU_PROLOGUE_CALLBACK_POP_ARG followed by the argument.
+	   */
 	void *prologue_callback_pop_arg;
 
 	/**
@@ -920,13 +955,6 @@ struct starpu_task
 	*/
 	unsigned regenerate:1;
 
-	/**
-	   @private
-	   This is only used for tasks that use multiformat handle.
-	   This should only be used by StarPU.
-	*/
-	unsigned mf_skip:1;
-
 	/**
 	   do not allocate a submitorder id for this task
 
@@ -936,13 +964,26 @@ struct starpu_task
 	*/
 	unsigned no_submitorder:1;
 
+	/**
+	   @private
+	   This is only used for tasks that use multiformat handle.
+	   This should only be used by StarPU.
+	*/
+	unsigned char mf_skip;
+
 	/**
 	   Whether the scheduler has pushed the task on some queue
 
 	   Set by StarPU.
 	*/
-	unsigned scheduled:1;
-	unsigned prefetched:1;
+	unsigned char scheduled;
+
+	/**
+	   Whether the scheduler has prefetched the task's data
+
+	   Set by StarPU.
+	*/
+	unsigned char prefetched;
 
 	/**
 	   Optional field. If the field
@@ -1096,11 +1137,12 @@ struct starpu_task
 	struct starpu_profiling_task_info *profiling_info;
 
 	/**
-	   This can be set to the number of floating points operations
-	   that the task will have to achieve. This is useful for
-	   easily getting GFlops curves from the tool
-	   <c>starpu_perfmodel_plot</c>, and for the hypervisor load
-	   balancing.
+	   The application can set this to the number of floating points
+	   operations that the task will have to achieve. StarPU will measure
+	   the time that the task takes, and divide the two to get the GFlop/s
+	   achieved by the task.  This will allow getting GFlops/s curves
+	   from the tool <c>starpu_perfmodel_plot</c>, and is useful for the
+	   hypervisor load balancing.
 
 	   With starpu_task_insert() and alike this can be specified thanks to
 	   ::STARPU_FLOPS followed by a double.
@@ -1108,7 +1150,7 @@ struct starpu_task
 
 	double flops;
 	/**
-	   Output field. Predicted duration of the task. This field is
+	   Output field. Predicted duration of the task in microseconds. This field is
 	   only set if the scheduling strategy uses performance
 	   models.
 
