@@ -1,7 +1,6 @@
 ! StarPU --- Runtime system for heterogeneous multicore architectures.
 !
-! Copyright (C) 2016                                     Inria
-! Copyright (C) 2016                                     Université de Bordeaux
+! Copyright (C) 2016-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
 !
 ! StarPU is free software; you can redistribute it and/or modify
 ! it under the terms of the GNU Lesser General Public License as published by
@@ -26,7 +25,7 @@ program nf_basic_ring
         integer(c_int),target :: token = 42
         integer(c_int) :: nloops = 32
         integer(c_int) :: loop
-        integer(c_int) :: tag
+        integer(c_int64_t) :: tag
         integer(c_int) :: world
         integer(c_int) :: src,dst
         type(c_ptr) :: token_dh, st
