@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2012                                Inria
- * Copyright (C) 2012-2013,2017                           CNRS
- * Copyright (C) 2010-2011,2013, 2018                           Université de Bordeaux
+ * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -18,25 +16,27 @@
 
 #include "socl.h"
 
-static void mapbuffer_task(void *args) {
+static void mapbuffer_task(void *args)
+{
 	command_map_buffer cmd = (command_map_buffer)args;
 
-  cl_event ev = command_event_get(cmd);
-  ev->prof_start = _socl_nanotime();
-  gc_entity_release(ev);
+	cl_event ev = command_event_get(cmd);
+	ev->prof_start = _socl_nanotime();
+	gc_entity_release(ev);
 
 	enum starpu_data_access_mode mode = (cmd->map_flags == CL_MAP_READ ? STARPU_R : STARPU_RW);
 
 	starpu_data_acquire_cb(cmd->buffer->handle, mode, command_completed_task_callback, cmd);
 }
 
-static struct starpu_codelet codelet_mapbuffer = {
-   .name = "SOCL_MAP_BUFFER"
+static struct starpu_codelet codelet_mapbuffer =
+{
+	.name = "SOCL_MAP_BUFFER"
 };
 
-cl_int command_map_buffer_submit(command_map_buffer cmd) {
-
-   gc_entity_retain(cmd);
+cl_int command_map_buffer_submit(command_map_buffer cmd)
+{
+	gc_entity_retain(cmd);
 
 	cpu_task_submit(cmd, mapbuffer_task, cmd, 0, 0, &codelet_mapbuffer, 0, NULL);
 
@@ -56,10 +56,9 @@ soclEnqueueMapBuffer(cl_command_queue cq,
 		     cl_event *       event,
 		     cl_int *         errcode_ret)
 {
-
 	command_map_buffer cmd = command_map_buffer_create(buffer, map_flags, offset, cb);
 
-   cl_event ev = command_event_get(cmd);
+	cl_event ev = command_event_get(cmd);
 
 	command_queue_enqueue(cq, cmd, num_events, events);
 
