@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2008-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -38,7 +38,7 @@ unsigned long _starpu_job_cnt = 0;
 #include <sys/thr.h>       /* for thr_self() */
 #endif
 
-static char _STARPU_PROF_FILE_USER[128];
+static char _starpu_prof_file_user[128];
 int _starpu_fxt_started = 0;
 int _starpu_fxt_willstart = 1;
 starpu_pthread_mutex_t _starpu_fxt_started_mutex = STARPU_PTHREAD_MUTEX_INITIALIZER;
@@ -100,10 +100,18 @@ static void _starpu_profile_set_tracefile(void)
 	if (!user)
 		user = "";
 
-	char suffix[128];
-	snprintf(suffix, sizeof(suffix), "prof_file_%s_%d", user, _starpu_id);
+	char suffix[64];
+	char *fxt_suffix = starpu_getenv("STARPU_FXT_SUFFIX");
+	if (!fxt_suffix)
+	{
+		snprintf(suffix, sizeof(suffix), "prof_file_%s_%d", user, _starpu_id);
+	}
+	else
+	{
+		snprintf(suffix, sizeof(suffix), "%s_%d", fxt_suffix, _starpu_id);
+	}
 
-	snprintf(_STARPU_PROF_FILE_USER, sizeof(_STARPU_PROF_FILE_USER), "%s%s", fxt_prefix, suffix);
+	snprintf(_starpu_prof_file_user, sizeof(_starpu_prof_file_user), "%s/%s", fxt_prefix, suffix);
 }
 
 void starpu_profiling_set_id(int new_id)
@@ -113,7 +121,7 @@ void starpu_profiling_set_id(int new_id)
 	_starpu_profile_set_tracefile();
 
 #ifdef HAVE_FUT_SET_FILENAME
-	fut_set_filename(_STARPU_PROF_FILE_USER);
+	fut_set_filename(_starpu_prof_file_user);
 #endif
 }
 
@@ -139,12 +147,27 @@ void starpu_fxt_stop_profiling()
 	fut_keychange(FUT_DISABLE, FUT_KEYMASKALL, threadid);
 }
 
-void _starpu_fxt_init_profiling(unsigned trace_buffer_size)
+int starpu_fxt_is_enabled()
+{
+	return starpu_get_env_number_default("STARPU_FXT_TRACE", 1);
+}
+
+#ifdef HAVE_FUT_SETUP_FLUSH_CALLBACK
+void _starpu_fxt_flush_callback()
+{
+	_STARPU_MSG("FxT is flushing trace to disk ! This can impact performance.\n");
+	_STARPU_MSG("Maybe you should increase the value of STARPU_TRACE_BUFFER_SIZE ?\n");
+
+	starpu_fxt_trace_user_event_string("fxt flush");
+}
+#endif
+
+void _starpu_fxt_init_profiling(uint64_t trace_buffer_size)
 {
 	unsigned threadid;
 
 	STARPU_PTHREAD_MUTEX_LOCK(&_starpu_fxt_started_mutex);
-	if (!(_starpu_fxt_willstart = starpu_get_env_number_default("STARPU_FXT_TRACE", 1)))
+	if (!(_starpu_fxt_willstart = starpu_fxt_is_enabled()))
 	{
 		STARPU_PTHREAD_COND_BROADCAST(&_starpu_fxt_started_cond);
 		STARPU_PTHREAD_MUTEX_UNLOCK(&_starpu_fxt_started_mutex);
@@ -158,7 +181,7 @@ void _starpu_fxt_init_profiling(unsigned trace_buffer_size)
 	_starpu_profile_set_tracefile();
 
 #ifdef HAVE_FUT_SET_FILENAME
-	fut_set_filename(_STARPU_PROF_FILE_USER);
+	fut_set_filename(_starpu_prof_file_user);
 #endif
 #ifdef HAVE_ENABLE_FUT_FLUSH
 	// when the event buffer is full, fxt stops recording events.
@@ -171,7 +194,11 @@ void _starpu_fxt_init_profiling(unsigned trace_buffer_size)
 
 	threadid = _starpu_gettid();
 
+#ifdef HAVE_FUT_SETUP_FLUSH_CALLBACK
+	if (fut_setup_flush_callback(trace_buffer_size / sizeof(unsigned long), initial_key_mask, threadid, &_starpu_fxt_flush_callback) < 0)
+#else
 	if (fut_setup(trace_buffer_size / sizeof(unsigned long), initial_key_mask, threadid) < 0)
+#endif
 	{
 		perror("fut_setup");
 		STARPU_ABORT();
@@ -261,10 +288,17 @@ void _starpu_fxt_dump_file(void)
 {
 	if (!_starpu_fxt_started)
 		return;
+
+	char hostname[128];
+	gethostname(hostname, 128);
+
+	int ret = fut_endup(_starpu_prof_file_user);
+	if (ret < 0)
+		_STARPU_MSG("Problem when writing FxT traces into file %s:%s\n", hostname, _starpu_prof_file_user);
 #ifdef STARPU_VERBOSE
-	_STARPU_MSG("Writing FxT traces into file %s\n", _STARPU_PROF_FILE_USER);
+	else
+		_STARPU_MSG("Writing FxT traces into file %s:%s\n", hostname, _starpu_prof_file_user);
 #endif
-	fut_endup(_STARPU_PROF_FILE_USER);
 }
 
 void _starpu_stop_fxt_profiling(void)
@@ -273,17 +307,12 @@ void _starpu_stop_fxt_profiling(void)
 		return;
 	if (!_starpu_written)
 	{
-#ifdef STARPU_VERBOSE
-	        char hostname[128];
-		gethostname(hostname, 128);
-		_STARPU_MSG("Writing FxT traces into file %s:%s\n", hostname, _STARPU_PROF_FILE_USER);
-#endif
-		fut_endup(_STARPU_PROF_FILE_USER);
+		_starpu_fxt_dump_file();
 
 		/* Should we generate a Paje trace directly ? */
 		int generate_trace = starpu_get_env_number("STARPU_GENERATE_TRACE");
 		if (generate_trace == 1)
-			_starpu_generate_paje_trace(_STARPU_PROF_FILE_USER, "paje.trace");
+			_starpu_generate_paje_trace(_starpu_prof_file_user, "paje.trace");
 
 		int ret = fut_done();
 		if (ret < 0)
