@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2013       Corentin Salingue
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -34,6 +34,12 @@
 #undef STARPU_UNISTD_USE_COPY
 #endif
 
+#ifdef __linux__
+typedef loff_t starpu_loff_t;
+#else
+typedef off_t starpu_loff_t;
+#endif
+
 struct starpu_unistd_global_obj
 {
         int descriptor;
