@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2013       Thibaut Lambert
  * Copyright (C) 2016       Uppsala University
  *
@@ -792,6 +792,12 @@ static void _starpu_init_topology(struct _starpu_machine_config *config)
 	_starpu_topology_filter(topology->hwtopology);
 	hwloc_topology_load(topology->hwtopology);
 
+#ifdef HAVE_HWLOC_CPUKINDS_GET_NR
+	int nr_kinds = hwloc_cpukinds_get_nr(topology->hwtopology, 0);
+	if (nr_kinds > 1)
+		_STARPU_DISP("Warning: there are several kinds of CPU on this system. For now StarPU assumes all CPU are equal\n");
+#endif
+
 	if (starpu_get_env_number_default("STARPU_WORKERS_GETBIND", 0))
 	{
 		/* Respect the existing binding */
@@ -874,9 +880,17 @@ static void _starpu_initialize_workers_bindid(struct _starpu_machine_config *con
 	unsigned i;
 
 	struct _starpu_machine_topology *topology = &config->topology;
+	int nhyperthreads = topology->nhwpus / topology->nhwcpus;
+	unsigned bind_on_core = 0;
+	int scale = 1;
 
 	config->current_bindid = 0;
 
+	if (starpu_getenv("STARPU_WORKERS_CPUID") && starpu_getenv("STARPU_WORKERS_COREID"))
+	{
+		_STARPU_DISP("Warning: STARPU_WORKERS_CPUID and STARPU_WORKERS_COREID cannot be set at the same time. STARPU_WORKERS_CPUID will be used.\n");
+	}
+
 	/* conf->workers_bindid indicates the successive logical PU identifier that
 	 * should be used to bind the workers. It should be either filled
 	 * according to the user's explicit parameters (from starpu_conf) or
@@ -886,6 +900,16 @@ static void _starpu_initialize_workers_bindid(struct _starpu_machine_config *con
 
 	/* what do we use, explicit value, env. variable, or round-robin ? */
 	strval = starpu_getenv("STARPU_WORKERS_CPUID");
+	if (strval == NULL)
+	{
+		strval = starpu_getenv("STARPU_WORKERS_COREID");
+		if (strval)
+		{
+			bind_on_core = 1;
+			scale = nhyperthreads;
+		}
+	}
+
 	if (strval)
 	{
 		/* STARPU_WORKERS_CPUID certainly contains less entries than
@@ -906,7 +930,7 @@ static void _starpu_initialize_workers_bindid(struct _starpu_machine_config *con
 				val = strtol(strval, &endptr, 10);
 				if (endptr != strval)
 				{
-					topology->workers_bindid[i] = (unsigned)(val % topology->nhwpus);
+					topology->workers_bindid[i] = (unsigned)((val * scale) % topology->nhwpus);
 					strval = endptr;
 					if (*strval == '-')
 					{
@@ -920,14 +944,14 @@ static void _starpu_initialize_workers_bindid(struct _starpu_machine_config *con
 						}
 						else
 						{
-							endval = topology->nhwpus-1;
+							endval = (bind_on_core ? topology->nhwcpus : topology->nhwpus) - 1;
 							if (*strval)
 								strval++;
 						}
 						for (val++; val <= endval && i < STARPU_NMAXWORKERS-1; val++)
 						{
 							i++;
-							topology->workers_bindid[i] = (unsigned)(val % topology->nhwpus);
+							topology->workers_bindid[i] = (unsigned)((val * scale) % topology->nhwpus);
 						}
 					}
 					if (*strval == ',')
@@ -953,6 +977,7 @@ static void _starpu_initialize_workers_bindid(struct _starpu_machine_config *con
 					topology->workers_bindid[i % number_of_entries];
 			}
 		}
+		topology->workers_nbindid = number_of_entries;
 	}
 	else if (config->conf.use_explicit_workers_bindid)
 	{
@@ -960,13 +985,13 @@ static void _starpu_initialize_workers_bindid(struct _starpu_machine_config *con
 		memcpy(topology->workers_bindid,
 			config->conf.workers_bindid,
 			STARPU_NMAXWORKERS*sizeof(unsigned));
+		topology->workers_nbindid = STARPU_NMAXWORKERS;
 	}
 	else
 	{
 		int nth_per_core = starpu_get_env_number_default("STARPU_NTHREADS_PER_CORE", 1);
 		int k;
 		int nbindids=0;
-		int nhyperthreads = topology->nhwpus / topology->nhwcpus;
 		STARPU_ASSERT_MSG(nth_per_core > 0 && nth_per_core <= nhyperthreads , "Incorrect number of hyperthreads");
 
 		i = 0; /* PU number currently assigned */
@@ -988,6 +1013,7 @@ static void _starpu_initialize_workers_bindid(struct _starpu_machine_config *con
 			k++;
 			i++;
 		}
+		topology->workers_nbindid = topology->nhwpus * nth_per_core / nhyperthreads;
 	}
 
 	for (i = 0; i < STARPU_MAXCPUS;i++)
@@ -1026,7 +1052,7 @@ static inline unsigned _starpu_get_next_bindid(struct _starpu_machine_config *co
 
 	unsigned current_preferred;
 	unsigned nhyperthreads = topology->nhwpus / topology->nhwcpus;
-	unsigned ncores = topology->nhwpus / nhyperthreads;
+	unsigned workers_nbindid = topology->workers_nbindid;
 	unsigned i;
 
 	if (npreferred)
@@ -1043,15 +1069,15 @@ static inline unsigned _starpu_get_next_bindid(struct _starpu_machine_config *co
 		unsigned requested_core = preferred_binding[current_preferred];
 		unsigned requested_bindid = requested_core * nhyperthreads;
 
-		/* Look at the remaining cores to be bound to */
-		for (i = 0; i < ncores; i++)
+		/* Look at the remaining PUs to be bound to */
+		for (i = 0; i < workers_nbindid; i++)
 		{
 			if (topology->workers_bindid[i] == requested_bindid &&
 					(!config->currently_bound[i] ||
 					 (config->currently_shared[i] && !(flags & STARPU_THREAD_ACTIVE)))
 					)
 			{
-				/* the cpu is available, or shareable with us, we use it ! */
+				/* the PU is available, or shareable with us, we use it ! */
 				config->currently_bound[i] = 1;
 				if (!(flags & STARPU_THREAD_ACTIVE))
 					config->currently_shared[i] = 1;
@@ -1063,18 +1089,18 @@ static inline unsigned _starpu_get_next_bindid(struct _starpu_machine_config *co
 	if (!(flags & STARPU_THREAD_ACTIVE))
 	{
 		/* Try to find a shareable PU */
-		for (i = 0; i < ncores; i++)
+		for (i = 0; i < workers_nbindid; i++)
 			if (config->currently_shared[i])
 				return topology->workers_bindid[i];
 	}
 
 	/* Try to find an available PU from last used PU */
-	for (i = config->current_bindid; i < ncores; i++)
+	for (i = config->current_bindid; i < workers_nbindid; i++)
 		if (!config->currently_bound[i])
 			/* Found a cpu ready for use, use it! */
 			break;
 
-	if (i == ncores)
+	if (i == workers_nbindid)
 	{
 		/* Finished binding on all cpus, restart from start in
 		 * case the user really wants overloading */
@@ -1082,7 +1108,7 @@ static inline unsigned _starpu_get_next_bindid(struct _starpu_machine_config *co
 		i = 0;
 	}
 
-	STARPU_ASSERT(i < ncores);
+	STARPU_ASSERT(i < workers_nbindid);
 	unsigned bindid = topology->workers_bindid[i];
 	config->currently_bound[i] = 1;
 	if (!(flags & STARPU_THREAD_ACTIVE))
@@ -1162,10 +1188,10 @@ unsigned _starpu_topology_get_nnumanodes(struct _starpu_machine_config *config S
 void _starpu_topology_filter(hwloc_topology_t topology)
 {
 #if HWLOC_API_VERSION >= 0x20000
-	hwloc_topology_set_io_types_filter(topology, HWLOC_TYPE_FILTER_KEEP_IMPORTANT);
+	hwloc_topology_set_io_types_filter(topology, HWLOC_TYPE_FILTER_KEEP_ALL);
 	hwloc_topology_set_flags(topology, HWLOC_TOPOLOGY_FLAG_IS_THISSYSTEM);
 #else
-	hwloc_topology_set_flags(topology, HWLOC_TOPOLOGY_FLAG_IS_THISSYSTEM | HWLOC_TOPOLOGY_FLAG_IO_DEVICES | HWLOC_TOPOLOGY_FLAG_IO_BRIDGES);
+	hwloc_topology_set_flags(topology, HWLOC_TOPOLOGY_FLAG_IS_THISSYSTEM | HWLOC_TOPOLOGY_FLAG_WHOLE_IO);
 #endif
 #ifdef HAVE_HWLOC_TOPOLOGY_SET_COMPONENTS
 #  ifndef STARPU_USE_CUDA
@@ -2970,8 +2996,8 @@ void starpu_topology_print(FILE *output)
 #ifdef STARPU_HAVE_HWLOC
 	hwloc_topology_t topo = topology->hwtopology;
 	hwloc_obj_t pu_obj;
-	hwloc_obj_t last_numa_obj = NULL, numa_obj;
-	hwloc_obj_t last_package_obj = NULL, package_obj;
+	hwloc_obj_t last_numa_obj = (void*) -1, numa_obj;
+	hwloc_obj_t last_package_obj = (void*) -1, package_obj;
 #endif
 
 	for (pu = 0; pu < topology->nhwpus; pu++)
@@ -2981,14 +3007,20 @@ void starpu_topology_print(FILE *output)
 		numa_obj = numa_get_obj(pu_obj);
 		if (numa_obj != last_numa_obj)
 		{
-			fprintf(output, "numa %u", numa_obj->logical_index);
+			if (numa_obj)
+				fprintf(output, "numa %2u", numa_obj->logical_index);
+			else
+				fprintf(output, "No numa");
 			last_numa_obj = numa_obj;
 		}
 		fprintf(output, "\t");
 		package_obj = hwloc_get_ancestor_obj_by_type(topo, HWLOC_OBJ_SOCKET, pu_obj);
 		if (package_obj != last_package_obj)
 		{
-			fprintf(output, "pack %u", package_obj->logical_index);
+			if (package_obj)
+				fprintf(output, "pack %2u", package_obj->logical_index);
+			else
+				fprintf(output, "no pack");
 			last_package_obj = package_obj;
 		}
 		fprintf(output, "\t");
@@ -3049,3 +3081,10 @@ hwloc_topology_t starpu_get_hwloc_topology(void)
 	return config->topology.hwtopology;
 }
 #endif
+
+unsigned _starpu_get_nhyperthreads()
+{
+	struct _starpu_machine_config *config = _starpu_get_machine_config();
+
+	return config->topology.nhwpus / config->topology.nhwcpus;
+}
