@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2014,2015                                Inria
- * Copyright (C) 2014-2017, 2019                          CNRS
- * Copyright (C) 2014,2016                                Université de Bordeaux
+ * Copyright (C) 2014-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -112,6 +110,9 @@ struct starpu_tree* starpu_tree_get_neighbour(struct starpu_tree *tree, struct s
 
 	if (father == NULL) return NULL;
 
+	if (father == tree && father->arity == 0)
+		return tree;
+
 	for(st = 0; st < father->arity; st++)
 	{
 		if(&father->nodes[st] == node)
