@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2008-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2011       Télécom-SudParis
  * Copyright (C) 2013       Thibaut Lambert
  * Copyright (C) 2016       Uppsala University
@@ -360,9 +360,18 @@ static inline int _starpu_can_use_nth_implementation(enum starpu_worker_archtype
 	return 0;
 }
 
-/* must be called with sched_mutex locked to protect state_blocked_in_parallel */
-int starpu_worker_can_execute_task(unsigned workerid, struct starpu_task *task, unsigned nimpl)
-{
+/* Test if this task can be processed on this worker, regardless of the implementation */
+/* must be called with sched_mutex locked to protect state_blocked */
+static inline int _starpu_can_execute_task_any_impl(unsigned workerid, struct starpu_task *task) {
+
+	if (task->workerids_len)
+	{
+		size_t div = sizeof(*task->workerids) * 8;
+		if (workerid / div >= task->workerids_len || ! (task->workerids[workerid / div] & (1UL << workerid % div)))
+			return 0;
+	}
+
+	
 	/* if the worker is blocked in a parallel ctx don't submit tasks on it */
 #ifdef STARPU_DEVEL
 #warning FIXME: this is very expensive, while can_execute is supposed to be not very costly so schedulers can call it a lot
@@ -370,8 +379,18 @@ int starpu_worker_can_execute_task(unsigned workerid, struct starpu_task *task,
 	if(starpu_worker_is_blocked_in_parallel(workerid))
 		return 0;
 
+	if (!(task->where & _starpu_config.workers[workerid].worker_mask))
+		return 0;
+
+	return 1; 
+}
+
+/* must be called with sched_mutex locked to protect state_blocked_in_parallel */
+int starpu_worker_can_execute_task(unsigned workerid, struct starpu_task *task, unsigned nimpl)
+{
+
 	/* TODO: check that the task operand sizes will fit on that device */
-	return (task->where & _starpu_config.workers[workerid].worker_mask) &&
+	return  _starpu_can_execute_task_any_impl(workerid, task) &&
 		_starpu_can_use_nth_implementation(_starpu_config.workers[workerid].arch, task->cl, nimpl) &&
 		(!task->cl->can_execute || task->cl->can_execute(workerid, task, nimpl));
 }
@@ -379,8 +398,7 @@ int starpu_worker_can_execute_task(unsigned workerid, struct starpu_task *task,
 /* must be called with sched_mutex locked to protect state_blocked_in_parallel */
 int starpu_worker_can_execute_task_impl(unsigned workerid, struct starpu_task *task, unsigned *impl_mask)
 {
-	/* if the worker is blocked in a parallel ctx don't submit tasks on it */
-	if(starpu_worker_is_blocked_in_parallel(workerid))
+	if (!_starpu_can_execute_task_any_impl(workerid, task))
 		return 0;
 
 	unsigned mask;
@@ -389,15 +407,6 @@ int starpu_worker_can_execute_task_impl(unsigned workerid, struct starpu_task *t
 	struct starpu_codelet *cl;
 	/* TODO: check that the task operand sizes will fit on that device */
 	cl = task->cl;
-	if (!(task->where & _starpu_config.workers[workerid].worker_mask))
-		return 0;
-
-	if (task->workerids_len)
-	{
-		size_t div = sizeof(*task->workerids) * 8;
-		if (workerid / div >= task->workerids_len || ! (task->workerids[workerid / div] & (1UL << workerid % div)))
-			return 0;
-	}
 
 	mask = 0;
 	arch = _starpu_config.workers[workerid].arch;
@@ -430,16 +439,13 @@ int starpu_worker_can_execute_task_impl(unsigned workerid, struct starpu_task *t
 /* must be called with sched_mutex locked to protect state_blocked */
 int starpu_worker_can_execute_task_first_impl(unsigned workerid, struct starpu_task *task, unsigned *nimpl)
 {
-	/* if the worker is blocked in a parallel ctx don't submit tasks on it */
-	if(starpu_worker_is_blocked_in_parallel(workerid))
+	if (!_starpu_can_execute_task_any_impl(workerid, task))
 		return 0;
 	int i;
 	enum starpu_worker_archtype arch;
 	struct starpu_codelet *cl;
 	/* TODO: check that the task operand sizes will fit on that device */
 	cl = task->cl;
-	if (!(task->where & _starpu_config.workers[workerid].worker_mask))
-		return 0;
 
 	arch = _starpu_config.workers[workerid].arch;
 	if (!task->cl->can_execute)
@@ -1293,7 +1299,12 @@ int starpu_initialize(struct starpu_conf *user_conf, int *argc, char ***argv)
 #ifdef STARPU_USE_MPI_MASTER_SLAVE
         if (_starpu_mpi_common_mp_init() == -ENODEV)
         {
-                initialized = UNINITIALIZED;
+		STARPU_PTHREAD_MUTEX_LOCK(&init_mutex);
+		init_count--;
+		initialized = UNINITIALIZED;
+		/* Let somebody else try to do it */
+		STARPU_PTHREAD_COND_SIGNAL(&init_cond);
+		STARPU_PTHREAD_MUTEX_UNLOCK(&init_mutex);
                 return -ENODEV;
         }
 
@@ -1372,6 +1383,12 @@ int starpu_initialize(struct starpu_conf *user_conf, int *argc, char ***argv)
 	if (starpu_get_env_number_default("STARPU_SIMGRID", 0))
 	{
 		_STARPU_DISP("Simulation mode requested, but this libstarpu was built without simgrid support, please recompile\n");
+		STARPU_PTHREAD_MUTEX_LOCK(&init_mutex);
+		init_count--;
+		initialized = UNINITIALIZED;
+		/* Let somebody else try to do it */
+		STARPU_PTHREAD_COND_SIGNAL(&init_cond);
+		STARPU_PTHREAD_MUTEX_UNLOCK(&init_mutex);
 		return -EINVAL;
 	}
 #endif
@@ -1391,6 +1408,12 @@ int starpu_initialize(struct starpu_conf *user_conf, int *argc, char ***argv)
 		if (user_conf->magic != 42)
 		{
 			_STARPU_DISP("starpu_conf structure needs to be initialized with starpu_conf_init\n");
+			STARPU_PTHREAD_MUTEX_LOCK(&init_mutex);
+			init_count--;
+			initialized = UNINITIALIZED;
+			/* Let somebody else try to do it */
+			STARPU_PTHREAD_COND_SIGNAL(&init_cond);
+			STARPU_PTHREAD_MUTEX_UNLOCK(&init_mutex);
 			return -EINVAL;
 		}
 		_starpu_config.conf = *user_conf;
@@ -1870,6 +1893,12 @@ unsigned starpu_worker_get_count(void)
 	return _starpu_config.topology.nworkers;
 }
 
+void starpu_worker_get_current_task_exp_end(unsigned workerid, struct timespec *date)
+{
+	struct _starpu_worker *worker = _starpu_get_worker_struct(workerid);
+	*date = worker->cl_expend;
+}
+
 unsigned starpu_worker_is_blocked_in_parallel(int workerid)
 {
 	if (!_starpu_worker_parallel_blocks)
