@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2008-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2010       Mehdi Juhoor
  * Copyright (C) 2013       Thibaut Lambert
  *
@@ -192,7 +192,7 @@ static void _starpu_data_partition(starpu_data_handle_t initial_handle, starpu_d
 		int home_node = initial_handle->home_node;
 		if (home_node < 0 || (starpu_node_get_kind(home_node) != STARPU_CPU_RAM))
 			home_node = STARPU_MAIN_RAM;
-		int ret = _starpu_allocate_memory_on_node(initial_handle, &initial_handle->per_node[home_node], 0);
+		int ret = _starpu_allocate_memory_on_node(initial_handle, &initial_handle->per_node[home_node], STARPU_FETCH);
 #ifdef STARPU_DEVEL
 #warning we should reclaim memory if allocation failed
 #endif
@@ -667,7 +667,10 @@ void starpu_data_partition_clean(starpu_data_handle_t root_handle, unsigned npar
 	free(children[0]->siblings);
 
 	for (i = 0; i < nparts; i++)
+	{
+		children[i]->siblings = NULL;
 		starpu_data_unregister_submit(children[i]);
+	}
 
 	_starpu_spin_lock(&root_handle->header_lock);
 	root_handle->nplans--;
@@ -718,7 +721,7 @@ void _starpu_data_partition_submit(starpu_data_handle_t initial_handle, unsigned
 					 0);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
 	if (!handles_sequential_consistency || handles_sequential_consistency[0])
-		starpu_data_invalidate_submit(initial_handle);
+		_starpu_data_invalidate_submit_noplan(initial_handle);
 }
 
 void starpu_data_partition_submit_sequential_consistency(starpu_data_handle_t initial_handle, unsigned nparts, starpu_data_handle_t *children, int sequential_consistency)
@@ -796,7 +799,7 @@ void starpu_data_partition_readwrite_upgrade_submit(starpu_data_handle_t initial
 	}
 	/* TODO: assert nparts too */
 	starpu_task_insert(initial_handle->switch_cl, STARPU_RW, initial_handle, STARPU_DATA_MODE_ARRAY, descr, nparts, 0);
-	starpu_data_invalidate_submit(initial_handle);
+	_starpu_data_invalidate_submit_noplan(initial_handle);
 }
 
 void _starpu_data_unpartition_submit(starpu_data_handle_t initial_handle, unsigned nparts, starpu_data_handle_t *children, int gather_node, unsigned char *handles_sequential_consistency, void (*callback_func)(void *), void *callback_arg)
@@ -868,7 +871,7 @@ void _starpu_data_unpartition_submit(starpu_data_handle_t initial_handle, unsign
 	for (i = 0; i < nparts; i++)
 	{
 		if (!handles_sequential_consistency || handles_sequential_consistency[i+1])
-			starpu_data_invalidate_submit(children[i]);
+			_starpu_data_invalidate_submit_noplan(children[i]);
 	}
 }
 
@@ -935,7 +938,7 @@ void starpu_data_unpartition_submit_r(starpu_data_handle_t ancestor, int gatheri
 		/* Uh, has to go through all read-only partitions */
 		for (i = 0; i < n; i++)
 		{
-			/* Note: active_readonly_children is emptied by starpu_data_unpartition_submit calls */
+			/* Note: active_readonly_children is emptied by starpu_data_unpartition_submit_r calls below, that's why we always refer to [0] here */
 			starpu_data_handle_t *children = ancestor->active_readonly_children[0];
 			_STARPU_DEBUG("unpartition readonly children %p etc.\n", children[0]);
 			nsiblings = children[0]->nsiblings;
@@ -1070,8 +1073,3 @@ starpu_filter_nparts_compute_chunk_size_and_offset(unsigned n, unsigned nparts,
 	if (offset != NULL)
 		*offset = (id *(n/nparts) + STARPU_MIN(remainder, id)) * ld * elemsize;
 }
-
-void starpu_data_partition_not_automatic(starpu_data_handle_t handle)
-{
-	handle->partition_automatic_disabled = 1;
-}
