@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013-2015,2017                           Inria
- * Copyright (C) 2017                                     CNRS
- * Copyright (C) 2014,2017,2018                           Université de Bordeaux
+ * Copyright (C) 2013-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -29,16 +27,10 @@ void starpu_initialize_prio_center_policy(unsigned sched_ctx_id)
 			STARPU_SCHED_SIMPLE_IMPL, sched_ctx_id);
 }
 
-static void deinitialize_prio_center_policy(unsigned sched_ctx_id)
-{
-	struct starpu_sched_tree *tree = (struct starpu_sched_tree*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
-	starpu_sched_tree_destroy(tree);
-}
-
 struct starpu_sched_policy _starpu_sched_modular_prio_policy =
 {
 	.init_sched = starpu_initialize_prio_center_policy,
-	.deinit_sched = deinitialize_prio_center_policy,
+	.deinit_sched = starpu_sched_tree_deinitialize,
 	.add_workers = starpu_sched_tree_add_workers,
 	.remove_workers = starpu_sched_tree_remove_workers,
 	.push_task = starpu_sched_tree_push_task,
