@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011,2013,2015                           CNRS
- * Copyright (C) 2011,2014-2015                           Université de Bordeaux
+ * Copyright (C) 2011-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -18,6 +18,8 @@
 #ifndef _STARPU_PARAMETERS_H
 #define _STARPU_PARAMETERS_H
 
+/** @file */
+
 /* Parameters which are not worth being added to ./configure options, but
  * still interesting to easily change */
 
@@ -27,5 +29,5 @@
 #define _STARPU_CUDA_ALPHA	13.33f
 #define _STARPU_OPENCL_ALPHA	12.22f
 #define _STARPU_MIC_ALPHA	0.5f
-#define _STARPU_SCC_ALPHA	1.0f
+#define _STARPU_MPI_MS_ALPHA	1.0f
 #endif /* _STARPU_PARAMETERS_H */
