@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2017                                Inria
- * Copyright (C) 2012-2017                                CNRS
- * Copyright (C) 2013,2014,2016,2017                      Université de Bordeaux
+ * Copyright (C) 2011-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -358,7 +356,7 @@ static void tree_init_iterator_for_parallel_tasks(struct starpu_worker_collectio
 	}
 }
 
-struct starpu_worker_collection worker_tree =
+struct starpu_worker_collection starpu_worker_tree =
 {
 	.has_next = tree_has_next,
 	.get_next = tree_get_next,
