@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2015,2017                           CNRS
- * Copyright (C) 2012,2013,2017                           Inria
- * Copyright (C) 2009-2012,2014,2019                      Université de Bordeaux
+ * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -243,7 +241,7 @@ int main(int argc, char *argv[])
 	gettimeofday(&begin, NULL);
 	if (cufftExecC2C(cuda_plan, (cufftComplex*) in, (cufftComplex*) out_cuda, CUFFT_FORWARD) != CUFFT_SUCCESS)
 		printf("erf2\n");
-	if ((cures = cudaThreadSynchronize()) != cudaSuccess)
+	if ((cures = cudaDeviceSynchronize()) != cudaSuccess)
 		STARPU_CUDA_REPORT_ERROR(cures);
 	gettimeofday(&end, NULL);
 	cufftDestroy(cuda_plan);
