@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2014,2016,2017                      Université de Bordeaux
- * Copyright (C) 2011-2013                                Inria
- * Copyright (C) 2010-2013,2015-2017                      CNRS
+ * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -222,7 +220,7 @@ static void *progress_func(void *arg)
 	progress_thread_running = 1;
 	STARPU_PTHREAD_COND_SIGNAL(&data_req_cond);
 
-	while (progress_thread_running)
+	while (progress_thread_running || data_req_list)
 	{
 		struct data_req *req;
 
@@ -238,6 +236,7 @@ static void *progress_func(void *arg)
 
 			STARPU_PTHREAD_MUTEX_UNLOCK(&data_req_mutex);
 
+			STARPU_VALGRIND_YIELD();
 			int ret = req->test_func(req->test_arg);
 
 			if (ret)
