@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010-2011, 2013-2014, 2016  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  CNRS
- * Copyright (C) 2011  CNRS
+ * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -35,8 +33,10 @@ static unsigned ntasks = 65536;
 #endif
 static unsigned nthreads = 2;
 
-void dummy_func(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg STARPU_ATTRIBUTE_UNUSED)
+void dummy_func(void *descr[], void *arg)
 {
+	(void)descr;
+	(void)arg;
 }
 
 static struct starpu_codelet dummy_codelet =
@@ -50,10 +50,11 @@ static struct starpu_codelet dummy_codelet =
 };
 
 static
-void *thread_func(void *arg STARPU_ATTRIBUTE_UNUSED)
+void *thread_func(void *arg)
 {
 	int ret;
 	unsigned i;
+	(void)arg;
 
 	for (i = 0; i < ntasks; i++)
 	{
@@ -119,14 +120,12 @@ int main(int argc, char **argv)
 	unsigned t;
 	for (t = 0; t < nthreads; t++)
 	{
-		ret = starpu_pthread_create(&threads[t], NULL, thread_func, NULL);
-		STARPU_ASSERT(ret == 0);
+		STARPU_PTHREAD_CREATE(&threads[t], NULL, thread_func, NULL);
 	}
 
 	for (t = 0; t < nthreads; t++)
 	{
-		ret = starpu_pthread_join(threads[t], NULL);
-		STARPU_ASSERT(ret == 0);
+		STARPU_PTHREAD_JOIN(threads[t], NULL);
 	}
 
 	end = starpu_timing_now();
