@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
- * Copyright (C) 2010-2016                                Université de Bordeaux
- * Copyright (C) 2010-2015                                CNRS
+ * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,6 +17,7 @@
 #include <starpu.h>
 #include <common/thread.h>
 
+#include "increment_codelet.h"
 #include "../helper.h"
 
 /*
@@ -63,31 +62,10 @@ static unsigned *check_cnt;
 static starpu_pthread_cond_t cond = STARPU_PTHREAD_COND_INITIALIZER;
 static starpu_pthread_mutex_t mutex = STARPU_PTHREAD_MUTEX_INITIALIZER;
 
-extern void cuda_host_increment(void *descr[], void *_args);
-
-void cpu_increment(void *descr[], void *arg STARPU_ATTRIBUTE_UNUSED)
+static void callback_task_A(void *arg)
 {
-	unsigned *var = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
-	(*var)++;
-}
+	(void)arg;
 
-static struct starpu_codelet dummy_codelet =
-{
-	.cpu_funcs = {cpu_increment},
-#ifdef STARPU_USE_CUDA
-	.cuda_funcs = {cuda_host_increment},
-	.cuda_flags = {STARPU_CUDA_ASYNC},
-#endif
-	// TODO
-	//.opencl_funcs = {dummy_func},
-	.cpu_funcs_name = {"cpu_increment"},
-	.model = NULL,
-	.modes = { STARPU_RW },
-	.nbuffers = 1
-};
-
-static void callback_task_A(void *arg STARPU_ATTRIBUTE_UNUSED)
-{
 	loop_cnt_A++;
 
 	if (loop_cnt_A == niter)
@@ -97,8 +75,10 @@ static void callback_task_A(void *arg STARPU_ATTRIBUTE_UNUSED)
 	}
 }
 
-static void callback_task_B(void *arg STARPU_ATTRIBUTE_UNUSED)
+static void callback_task_B(void *arg)
 {
+	(void)arg;
+
 	loop_cnt_B++;
 
 	if (loop_cnt_B == niter)
@@ -108,8 +88,10 @@ static void callback_task_B(void *arg STARPU_ATTRIBUTE_UNUSED)
 	}
 }
 
-static void callback_task_C(void *arg STARPU_ATTRIBUTE_UNUSED)
+static void callback_task_C(void *arg)
 {
+	(void)arg;
+
 	loop_cnt_C++;
 
 	if (loop_cnt_C == niter)
@@ -119,8 +101,10 @@ static void callback_task_C(void *arg STARPU_ATTRIBUTE_UNUSED)
 	}
 }
 
-static void callback_task_D(void *arg STARPU_ATTRIBUTE_UNUSED)
+static void callback_task_D(void *arg)
 {
+	(void)arg;
+
 	STARPU_PTHREAD_MUTEX_LOCK(&mutex);
 	loop_cnt++;
 
@@ -162,7 +146,7 @@ int main(int argc, char **argv)
 	starpu_variable_data_register(&check_data, STARPU_MAIN_RAM, (uintptr_t)check_cnt, sizeof(*check_cnt));
 
 	starpu_task_init(&taskA);
-	taskA.cl = &dummy_codelet;
+	taskA.cl = &increment_codelet;
 	taskA.regenerate = 1; /* this task will be explicitely resubmitted if needed */
 	taskA.use_tag = 1;
 	taskA.tag_id = TAG_A;
@@ -170,7 +154,7 @@ int main(int argc, char **argv)
 	taskA.handles[0] = check_data;
 
 	starpu_task_init(&taskB);
-	taskB.cl = &dummy_codelet;
+	taskB.cl = &increment_codelet;
 	taskB.regenerate = 1;
 	taskB.use_tag = 1;
 	taskB.tag_id = TAG_B;
@@ -178,7 +162,7 @@ int main(int argc, char **argv)
 	taskB.handles[0] = check_data;
 
 	starpu_task_init(&taskC);
-	taskC.cl = &dummy_codelet;
+	taskC.cl = &increment_codelet;
 	taskC.regenerate = 1;
 	taskC.use_tag = 1;
 	taskC.tag_id = TAG_C;
@@ -186,7 +170,7 @@ int main(int argc, char **argv)
 	taskC.handles[0] = check_data;
 
 	starpu_task_init(&taskD);
-	taskD.cl = &dummy_codelet;
+	taskD.cl = &increment_codelet;
 	taskD.callback_func = callback_task_D;
 	taskD.regenerate = 1;
 	taskD.use_tag = 1;
