@@ -1,7 +1,7 @@
 #!/bin/sh
 # StarPU --- Runtime system for heterogeneous multicore architectures.
 #
-# Copyright (C) 2009-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+# Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
 #
 # StarPU is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -19,9 +19,9 @@ ROOT=${ROOT%_sched}
 unset STARPU_SSILENT
 $STARPU_LAUNCH $_STARPU_LAUNCH $ROOT "$@" > tasks_size_overhead.output
 ret=$?
-if test "$ret" = "0"
+if test "$ret" = "0" && [ -z "$(echo $MAKEFLAGS | sed -ne 's/.*-j\([0-9]\+\).*/\1/p')" ]
 then
-    # if the program was successfull try to run gnuplot
+    # if the program was successfull and we are not running in parallel, try to run gnuplot
     DIR=
     [ -z "$STARPU_BENCH_DIR" ] || DIR="$STARPU_BENCH_DIR/"
     export TERMINAL=png
