@@ -1,8 +1,9 @@
 #!/usr/bin/env python3
-# -*- coding: utf-8 -*-
+# coding=utf-8
+#
 # StarPU --- Runtime system for heterogeneous multicore architectures.
 #
-# Copyright (C) 2019                                      CNRS
+# Copyright (C) 2019-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
 #
 # StarPU is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -32,12 +33,12 @@ def usage():
     print("	-v, --version       output version information and exit")
     print("	-png                produce plots in png format (default is pdf)")
     print("")
-    print("Report bugs to <starpu-devel@lists.gforge.inria.fr>")
+    print("Report bugs to <starpu-devel@inria.fr>")
     sys.exit(1)
 
 if len(sys.argv) >= 2:
     if sys.argv[1] == '-v' or sys.argv[1] == '--version':
-        print("%s (StarPU) 1.3.2" % PROGNAME)
+        print("%s (StarPU) 1.3.10" % PROGNAME)
         sys.exit(0)
     if sys.argv[1] == '-h' or sys.argv[1] == '--help':
         usage()
